#!/Library/Frameworks/EPD64.framework/Versions/Current/bin/python

"""
File: Salix_dadi_models.py
Author: Michael G. Harvey

Created by Michael G. Harvey on 21 September 2012
Copyright (c) 2012 Michael G. Harvey. All rights reserved.

Description: demographic models for Salix melanopsis in the Rocky Mountains only

"""

import numpy
import dadi

"""

Parameters used in models:

nuA: The ancestral population size
Te: The time to the population expansions
T1: The time between the more recent split and the population expansion
T2: The time between the older split and the more recent split
m1-m6: Migration parameters
nu1: Present-day population size of population 1
nu2: Present-day population size of population 2
nu3: Present-day population size of population 3
    
Note that unless otherwise specified:
    
population 1 = Salmon Drainage
population 2 = Clearwater Drainage
population 3 = Northern Drainages
    
n1,n2,n2: Size of fs to generate.
pts: Number of points to use in grid for evaluation.
    
"""

    
"""

NSC-all

Like SCN-all but with N basal to other populations.

Note that for only this model:
    
population 1 = Northern Drainages
population 2 = Clearwater Drainage
population 3 = Salmon Drainage
    
This model uses a different empirical frequency spectrum than the other models.
    
"""

def NSC-all((nuA, Te, T1, T2, m1, m2, m3, nu1, nu2, nu3), (n1,n2,n3), pts):



    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
  
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    nu3_func = lambda t: nuA*(nu3/nuA)**(t/Te)    
    
    # Integration with expansion and migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1_func, nu2_func, nu3_func, 
                                    m12=m1, m13=m2, m21=m1, m23=m3, m31=m2, m32=m3)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-all

Desciption: Includes split between Salmon and (Clearwater + Northern Drainages), 
split between Clearwater and Northern Drainages, exponential increase 
in daughter populations, and migration between daughter populations.
   
"""

def SCN-all((nuA, Te, T1, T2, m1, m2, m3, nu1, nu2, nu3), (n1,n2,n3), pts):


    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
  
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    nu3_func = lambda t: nuA*(nu3/nuA)**(t/Te)    
    
    # Integration with expansion and migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1_func, nu2_func, nu3_func, 
                                    m12=m1, m13=m2, m21=m1, m23=m3, m31=m2, m32=m3)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs 


"""

CSN-all

Like SCN-all, but with C basal to other populations.

"""

def CSN-all((nuA, Te, T1, T2, m1, m2, m3, nu1, nu2, nu3), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_1(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
  
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    nu3_func = lambda t: nuA*(nu3/nuA)**(t/Te)    
    
    # Integration with expansion and migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1_func, nu2_func, nu3_func, 
                                    m12=m1, m13=m2, m21=m1, m23=m3, m31=m2, m32=m3)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-Brunsfeld

Like SCN-all, but with no increases and no migration into Northern Drainages from Salmon.

"""

def SCN-Brunsfeld((nuA, Te, T1, T2, m1, m2, nu1, nu2, nu3), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
          
    # Integration with migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1, nu2, nu3, 
                                    m12=m1, m13=m2, m21=m1, m23=0, m31=m2, m32=0)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-Miller

Like SCN-all, but with no increases and no migration into Northern Drainages from Clearwater.

"""

def SCN-Miller((nuA, Te, T1, T2, m1, m2, nu1, nu2, nu3), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
          
    # Integration with migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1, nu2, nu3, 
                                    m12=m1, m13=0, m21=m1, m23=m2, m31=0, m32=m2)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-Carstens

Like SCN-all, but asymmetric migration between Salmon and Clearwater and no migration from Northern Drainages into Salmon and Clearwater.
    
"""

def SCN-Carstens((nuA, Te, T1, T2, m1, m2, m3, m4, nu1, nu2, nu3), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
    
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    nu3_func = lambda t: nuA*(nu3/nuA)**(t/Te)
      
    # Integration with migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1_func, nu2_func, nu3_func, 
                                    m12=m1, m13=m2, m21=m3, m23=m4, m31=0, m32=0)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-noincrease

Like SCN-all, but no population increases.

"""

def SCN-noincrease((nuA, Te, T1, T2, m1, m2, m3, nu1, nu2, nu3), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
          
    # Integration with migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1, nu2, nu3, 
                                    m12=m1, m13=m2, m21=m1, m23=m3, m31=m2, m32=m3)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-noincrease_equaltheta_noM

Like SCN-all, but no population increases, thetas equal, and no migration.

"""

def SCN-noincrease_equaltheta_noM((nuA, Te, T1, T2, nu1), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2=nu1, nu3=nu1, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)

    # Integration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1, nu2=nu1, nu3=nu1, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-equaltheta

Like SCN-all, but thetas equal.

"""

def SCN-equaltheta((nuA, Te, T1, T2, m1, m2, m3, nu1), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2=nu1, nu3=nu1, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)

    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
          
    # Integration with expansion and migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1=nu1_func, nu2=nu1, nu3=nu1, 
                                    m12=m1, m13=m2, m21=m1, m23=m3, m31=m2, m32=m3)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-noincrease-equaltheta

Like SCN-all, but no increases and thetas equal.

"""

def SCN-noincrease_equaltheta((nuA, Te, T1, T2, m1, m2, m3, nu1), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2=nu1, nu3=nu1, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)

    # Integration with migration
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1, nu2=nu1, nu3=nu1, 
                                    m12=m1, m13=m2, m21=m1, m23=m3, m31=m2, m32=m3)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs


"""

SCN-noincrease-noM

Like SCN-all, but no increases and no migration.

"""

def SCN-noincrease_noM((nuA, T1, T2, nu1, nu2, nu3), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration 
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2, nu3, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs
    

"""

SCN-noM_equaltheta

Like SCN-all, but thetas equal and no migration.

"""

def SCN-noM_equaltheta((nuA, Te, T1, T2, nu1), (n1,n2,n3), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    
    # Integration
    phi = dadi.Integration.two_pops(phi, xx, T2, nu1, nu2=nu1, m12=0, m21=0)
    
    # The second divergence
    phi = dadi.PhiManip.phi_2D_to_3D_split_2(xx, phi)
    
    # Integration pre-expansion, with no migration
    phi = dadi.Integration.three_pops(phi, xx, T1, nu1, nu2=nu1, nu3=nu1, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)
    
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    nu3_func = lambda t: nuA*(nu3/nuA)**(t/Te)
      
    # Integration with expansion 
    phi = dadi.Integration.three_pops(phi, xx, Te, nu1=nu1_func, nu2=nu1, nu3=nu1, 
                                    m12=0, m13=0, m21=0, m23=0, m31=0, m32=0)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2,n3), (xx,xx,xx))
    return sfs
    
    
"""

two_pop_CN_S

Two population model with Clearwater and Northern Drainages combined. 
Includes asymmetric migration, increases, and unequal thetas.

"""

def two_pop_CN_S((nuA, Te, T1, m1, m2, nu1, nu2), (n1,n2), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)

    # Integration, pre-expansion, without migration
    phi = dadi.Integration.two_pops(phi, xx, T1, nu1=nu1, nu2=nu2, m12=0, m21=0)
    
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    
    # Integration with expansion and migration
    phi = dadi.Integration.two_pops(phi, xx, Te, nu1=nu1_func, nu2=nu2_func, m12=m1, m21=m2)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2), (xx,xx))
    return sfs
        
        
"""

two_pop_N_CS

Two population model with Clearwater and Salmon combined. 
Includes asymmetric migration, increases, and unequal thetas.

"""

def two_pop_N_CS((nuA, Te, T1, m1, m2, nu1, nu2), (n1,n2), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)

    # Integration, pre-expansion, without migration
    phi = dadi.Integration.two_pops(phi, xx, T1, nu1=nu1, nu2=nu2, m12=0, m21=0)
    
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    
    # Integration with expansion and migration
    phi = dadi.Integration.two_pops(phi, xx, Te, nu1=nu1_func, nu2=nu2_func, m12=m1, m21=m2)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2), (xx,xx))
    return sfs


"""

two_pop_C_NS

Two population model with Northern Drainages and Salmon combined. 
Includes asymmetric migration, increases, and unequal thetas.

"""

def two_pop_C_NS((nuA, Te, T1, m1, m2, nu1, nu2), (n1,n2), pts):

    # Define the grid we'll use
    xx = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population; won't integrate for this one
    phi = dadi.PhiManip.phi_1D(xx)

    # The first divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)

    # Integration, pre-expansion, without migration
    phi = dadi.Integration.two_pops(phi, xx, T1, nu1=nu1, nu2=nu2, m12=0, m21=0)
    
    # Non-constant population size formulas
    nu1_func = lambda t: nuA*(nu1/nuA)**(t/Te)
    nu2_func = lambda t: nuA*(nu2/nuA)**(t/Te)
    
    # Integration with expansion and migration
    phi = dadi.Integration.two_pops(phi, xx, Te, nu1=nu1_func, nu2=nu2_func, m12=m1, m21=m2)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2), (xx,xx))
    return sfs
                
          
